﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Linq;
using System.IO;

namespace KuduRestClient
{
    class KuduClient
    {
        CookieContainer _cookies;
        HttpClient _httpClient;
        string _baseUrl;
        string _basicAuth;

        public KuduClient(string siteName, string userAccount, string userPassword)
        {
            HttpClientHandler handler = new HttpClientHandler();
            _cookies = new CookieContainer();
            handler.CookieContainer = _cookies;

            HttpClient hc = new HttpClient(handler);
            _httpClient = hc;

            _basicAuth = string.Format("Basic {0}", ToBase64(userAccount + ":" + userPassword));
            _httpClient.DefaultRequestHeaders.Add("Authorization", _basicAuth);

            _baseUrl = $"https://{siteName}.scm.azurewebsites.net/";
        }

        private string ToBase64(string text)
        {
            byte [] buf = Encoding.ASCII.GetBytes(text);
            return Convert.ToBase64String(buf);
        }

        public async Task<ScmInfo> GetScmInfoAsync()
        {
            string url = $"{_baseUrl}api/scm/info";

            string text = await _httpClient.GetStringAsync(url);
            return JsonConvert.DeserializeObject<ScmInfo>(text);
        }

        public async Task<List<VfsFile>> GetFileListAsync(string directory)
        {
            string url = $"{_baseUrl}api/vfs/{directory}/";
            string text = await _httpClient.GetStringAsync(url);

            List<VfsFile> files = new List<VfsFile>();
            files.AddRange(JsonConvert.DeserializeObject<VfsFile[]>(text).Where ( (item) => item.mime != "inode/directory") );

            return files;
        }

        public async Task<string> GetFileAsStringAsync(VfsFile file)
        {
            return await _httpClient.GetStringAsync(file.href);
        }

        public async Task<FileContent> GetFileWithEtagAsync(VfsFile file)
        {
            HttpResponseMessage hrm = await _httpClient.GetAsync(file.href);
            hrm.Headers.TryGetValues("ETag", out var values);

            FileContent fc = new FileContent();

            fc.etag = values.FirstOrDefault();
            if (fc.etag != null)
            {
                fc.etag = fc.etag.Trim('\"');
            }

            fc.text = await hrm.Content.ReadAsStringAsync();

            return fc;
        }

        public async Task<MessageResponse> DeleteFileAsync(VfsFile file, string etag = "*")
        {
            _httpClient.DefaultRequestHeaders.Add("If-Match", "*");

            HttpResponseMessage hrm = await _httpClient.DeleteAsync(file.href);

            string text = await hrm.Content.ReadAsStringAsync();
            MessageResponse mr = JsonConvert.DeserializeObject<MessageResponse>(text);
            mr.HttpResultCode = hrm.StatusCode;

            return mr;
        }

        public async Task<List<SiteExtensionItem>> GetInstalledSiteExtensionListAsync(string filter = null)
        {
            string url = $"{_baseUrl}api/siteextensions";

            if (string.IsNullOrEmpty(filter) == false)
            {
                url = url + "?filter=" + filter;
            }

            string text = await _httpClient.GetStringAsync(url);

            List<SiteExtensionItem> list = new List<SiteExtensionItem>();
            list.AddRange(Newtonsoft.Json.JsonConvert.DeserializeObject<SiteExtensionItem[]>(text));

            return list;
        }

        public async Task<SiteExtensionItem> GetInstalledSiteExtensionAsync(string packageId)
        {
            string url = $"{_baseUrl}api/siteextensions/{packageId}";
            string text = await _httpClient.GetStringAsync(url);

            return Newtonsoft.Json.JsonConvert.DeserializeObject<SiteExtensionItem>(text);
        }

        public async Task<MessageResponse> DeleteInstalledSiteExtensionAsync(string packageId)
        {
            string url = $"{_baseUrl}api/siteextensions/{packageId}";
            HttpResponseMessage hrm = await _httpClient.DeleteAsync(url);

            string text = await hrm.Content.ReadAsStringAsync();

            MessageResponse mr = new MessageResponse();
            mr.Message = text;
            mr.HttpResultCode = hrm.StatusCode;

            return mr;
        }

        public async Task<MessageResponse> InstallSiteExtension(string packageId)
        {
            string url = $"{_baseUrl}api/siteextensions/{packageId}";

            HttpResponseMessage hrm = await _httpClient.PutAsync(url, null);

            string text = await hrm.Content.ReadAsStringAsync();
            MessageResponse mr = Newtonsoft.Json.JsonConvert.DeserializeObject<MessageResponse>(text);
            mr.HttpResultCode = hrm.StatusCode;

            return mr;
        }

        public async Task<CommandOutput> RunCommandAsync(string command, string dir)
        {
            string url = $"{_baseUrl}api/command";

            CommandInfo ci = new CommandInfo();
            ci.command = command;
            ci.dir = dir;

            string jsonText = Newtonsoft.Json.JsonConvert.SerializeObject(ci);

            StringContent sc = new StringContent(jsonText, Encoding.UTF8, "application/json");
            HttpResponseMessage hrm = await _httpClient.PostAsync(url, sc);

            string text = await hrm.Content.ReadAsStringAsync();
            CommandOutput output = Newtonsoft.Json.JsonConvert.DeserializeObject<CommandOutput>(text);
            output.Result = hrm.StatusCode == HttpStatusCode.OK;

            return output;
        }

        public async Task<ProcessInfo> GetMainSiteInfoAsync()
        {
            string url = $"{_baseUrl}api/processes/-1";

            HttpResponseMessage hrm = await _httpClient.GetAsync(url);

            string text = await hrm.Content.ReadAsStringAsync();
            return Newtonsoft.Json.JsonConvert.DeserializeObject<ProcessInfo>(text);
        }

        public async Task<MessageResponse> RestartMainSiteAsync()
        {
            string url = $"{_baseUrl}api/processes/-1";
            HttpResponseMessage hrm = await _httpClient.DeleteAsync(url);

            string text = await hrm.Content.ReadAsStringAsync();
            MessageResponse mr = Newtonsoft.Json.JsonConvert.DeserializeObject<MessageResponse>(text);
            mr.HttpResultCode = hrm.StatusCode;

            return mr;
        }

    }
}
